Extractors module for type-safe request data extraction.
Structs
- Accept
- Accept header marker.
- AppState
- Application state container.
- Authorization
- Authorization header marker.
- Background
Tasks - Background task queue for running tasks after response is sent.
- Basic
Auth - Extracts HTTP Basic authentication credentials from the
Authorizationheader. - Bearer
Token - Simple HTTP bearer token extractor.
- Content
Type - Content-Type header marker.
- Cookie
- A cookie to set in the response.
- Header
- Header extractor for individual HTTP headers.
- Header
Values - Multiple header values extractor.
- Host
- Host header marker.
- Json
- JSON body extractor.
- Json
Config - Configuration for JSON request body extraction.
- Named
Header - Named header extractor with explicit header name.
- OAuth2
Password Bearer - OAuth2 password bearer security scheme extractor.
- Page
- Paginated response wrapper.
- Pagination
- Pagination query parameters extractor.
- Pagination
Config - Configuration for pagination behavior.
- Path
- Path parameter extractor.
- Path
Params - Extracted path parameters stored in request extensions.
- Query
- Query string extractor.
- Query
Params - Stored query parameters for extraction.
- Request
Ref - Read-only request data access.
- Response
Mut - Mutable response wrapper for setting headers and cookies.
- Response
Mutations - Mutable response container for setting response headers and cookies.
- State
- State extractor for application-wide shared state.
- User
Agent - User-Agent header marker.
- XRequest
Id - X-Request-Id header marker.