Prelude module for convenient imports.
Structs
- App
- A configured web application.
- AppBuilder
- Builder for constructing an
App. - AppConfig
- Application configuration for the FastAPI Rust framework.
- Basic
Auth - Extracts HTTP Basic authentication credentials from the
Authorizationheader. - Bearer
Token - Simple HTTP bearer token extractor.
- Cookie
- A cookie to set in the response.
- Cors
- CORS middleware.
- Cors
Config - Cross-Origin Resource Sharing (CORS) configuration.
- Default
Dependency Config - Default dependency configuration (cache per request).
- Dependency
Overrides - Dependency override registry (primarily for testing).
- Depends
- Dependency injection extractor.
- Header
- Header extractor for individual HTTP headers.
- Http
Error - HTTP error that produces a response.
- Json
- JSON body extractor.
- NoCache
- Disable caching for this dependency.
- OAuth2
Password Bearer - OAuth2 password bearer security scheme extractor.
- OpenApi
- OpenAPI 3.1 document.
- Open
ApiBuilder - OpenAPI document builder.
- Page
- Paginated response wrapper.
- Pagination
- Pagination query parameters extractor.
- Path
- Path parameter extractor.
- Query
- Query string extractor.
- Request
- HTTP request.
- Request
Context - Request context that wraps asupersync’s capability context.
- Request
Id - A request ID that was extracted or generated for the current request.
- Request
IdMiddleware - Middleware that adds unique request IDs to requests and responses.
- Response
- HTTP response.
- Route
- A route definition with handler for request processing.
- Router
- Radix trie router.
- Server
- Synchronous HTTP server for request/response conversion.
- Server
Config - Server configuration for the HTTP/1.1 server.
- State
- State extractor for application-wide shared state.
- Status
Code - HTTP status code.
- Validation
Error - A single validation error item.
- Validation
Errors - Collection of validation errors (422 Unprocessable Entity response).
Enums
- Config
Error - Configuration loading errors.
- Dependency
Scope - Dependency resolution scope.
- Method
- HTTP method.
Traits
- Depends
Config - Configuration for
Dependsresolution. - Deserialize
- A data structure that can be deserialized from any data format supported by Serde.
- From
Dependency - Trait for types that can be injected as dependencies.
- From
Request - Trait for types that can be extracted from a request.
- Into
Response - Trait for types that can be converted into a response.
- Serialize
- A data structure that can be serialized into any data format supported by Serde.
Functions
- serve
- Convenience function to serve an App on the given address.
Type Aliases
- Default
Config - Backwards-friendly alias for the default config.
Attribute Macros
- delete
- Mark a function as a DELETE handler.
- get
- Mark a function as a GET handler.
- head
- Mark a function as a HEAD handler.
- options
- Mark a function as an OPTIONS handler.
- patch
- Mark a function as a PATCH handler.
- post
- Mark a function as a POST handler.
- put
- Mark a function as a PUT handler.
Derive Macros
- Deserialize
- Json
Schema - Derive JSON Schema for OpenAPI.
- Serialize
- Validate
- Derive validation for a struct.